#include <VRS51L3074_SDCC.h>


#define FIRLEN 16

// FIR Filter Coefficients
// FSAMPLE 39062HZ, N=16, LOW PASS 1KHZ -35DB @ 5KHZ

int fircoef[] = {
0xFE05,  //-0.0154821
0xFF2E,  //-0.0064096
0x00CC,  // 0.0062169
0x041B,  // 0.0320849
0x08DA,  // 0.0691436
0x0E2A,  // 0.1106661
0x12C8,  // 0.1467445
0x1579,  // 0.1677631
0x1579,  // 0.1677631
0x12C8,  // 0.1467445
0x0E2A,  // 0.1106661
0x08DA,  // 0.0691436
0x041B,  // 0.0320849
0x00CC,  // 0.0062169
0xFF2E,  //-0.0064096
0xFE05   //-0.0154821
};

__idata int datastack[FIRLEN];
unsigned char FIRon=1;


void initAD(void);
void initPWM(void);
void PWMTMRInterrupt(void) interrupt 13 ;

void main (void) {

	PERIPHEN2 |= (1<<5);				//Enable the Arithmetic Unit
    initAD();
    initPWM();
        P2&=~0x1F;		// Make sure Outputs are Low
        P2PINCFG&=~0x1F;        // 11100000 P2.0,1,2,3,4 = output
    GENINTEN = 0x03; //Enable global interrupt

    while (1){
    }
}


/* 
   This function destroys A, R0, R1, R2; be sure to save registers (if needed) before calling
   dat -> data buffer, data in 1.15 format
   coef -> coefficient table in 1.15 format
   len: filter lenght, in taps
   result: current out data in 1.15 format
*/

int FIR(__idata int *dat,__idata int *coef,unsigned char len) __naked
{
  dat=dat;
  coef=coef;
  len=len;
  
	_asm

	orl _DEVMEMCFG,#0x01				; Switch to SFR Page 1

	mov _AUCONFIG1,#0x08				; CAPREV  = 0  Previous Res capture is automatic
								; CAPMODE = 0  Capture of previous Result
								;		occurs when AUA0 is written into
								; OVCAPEN = 0  Capture on OV32 dis disabled
								; READCAP = 0  AURES contains current operation result
								; ADDSRC  = 10 Add SRC = AUPREV
								; MULCMD  = 00 Mul cmd = AUA x AUB

	mov _AUCONFIG2,#0xA0				; Clear the Arithmetic Unit registers
        mov _AUSHIFTCFG,#0x00                           ; Disable barrel shifter

	MOV R0,_FIR_PARM_2				; Copy Start address of FIR Coefficient Table into R0
	MOV R1,dpl					; Copy Start address of Data buffer into R1
	MOV R2,_FIR_PARM_3
  FIRloop:
		MOV  _AUA0,@R0				; copy LSB of pointed coefficient to AUA0
		INC  R0                                 ; AUA0 write triggers capture of last result
		MOV  _AUA1,@R0				; copy MSB of pointed coefficient to AUA1
		INC  R0
		MOV  _AUB0,@R1				; copy LSB of pointed coefficient to AUB0
		INC  R1
		MOV  _AUB1,@R1				; copy MSB of pointed coefficient to AUB1
		INC  R1                                 ; mult/acc operation every cycle
		DJNZ R2,FIRloop                         ; the accumulated data is in 2.30 format

	MOV R2,_FIR_PARM_3                              ; Filter length
        DEC R2                                          ; -1
        MOV A,R2
        ADD A,R2
        INC A
	ADD A,dpl                                        ; point to MSB of last element
	MOV R1,A
	DEC A
        DEC A
	MOV R0,A                                        ; point to MSB of n-1 element
  FIRshift:
		mov  A,@R0                             ; shift data in buffer instead of moving pointers
		mov  @R1,A                             ; this avoids dealing with a circular buffer
		dec  R0
		dec  R1
		mov  A,@R0
		mov  @R1,A
		dec  R0
		dec  R1
		DJNZ R2,FIRshift

        mov _AUSHIFTCFG,#0x01                           ; Set the barrel shifter to perform a 1-bit left shift
                                                        ; to get a 1.31 result
        mov dph,_AURES3                            ; get a 1.15 result in dp, discarding the 16 LSBs
        mov dpl,_AURES2

	anl _DEVMEMCFG,#0xFE                       ; Switch to SFR Page 0
        ret  
	_endasm;
}


void ADInterrupt(void) interrupt 2 __naked using 1
{

_asm
  setb P2.0
        push psw
        push dpl
        push dph
        push ACC
        mov psw,#0x8                 ; select bank 1
	// Read SPI data, process it and put it in PWMMID
        mov dph,_SPIRXTX1
        mov dpl,_SPIRXTX0
        anl dph,#0x0F                ; dp = adcdata, 12-bit offset binary
        mov a,dph                    ; adcdata <<= 4
        swap a                       ; copied from sdcc
        anl a,#0xf0
        xch a,dpl
        swap a
        xch a,dpl
        xrl a,dpl
        xch a,dpl
        anl a,#0xf0
        xch a,dpl
        xrl a,dpl                    ; 16-bit offset binary
        xrl a,#0x80                  ; convert from offset binary to 2s-complement in 1.15
        mov dph,a
        
	mov R0,#_datastack
        mov @R0,dpl
        inc R0
        mov @R0,dph                  ; datastack[0] = adcdata;
        mov A,_FIRon
        jz firoff
  setb P2.1
        mov dpl,#_datastack
        mov _FIR_PARM_2,#_fircoef
        mov _FIR_PARM_3,#FIRLEN
        lcall _FIR                    ; adcdata=FIR(datastack,fircoef,FIRLEN);
  clr P2.1
firoff:
        mov A,dph                     ; discard LSBs, get only 8-bit data
        xrl A,#0x80                   ; convert to 8-bit offset binary
                                      ; if(adcdata==0)    // correct PWM flaw
	jnz no0
	inc A	                  ; adcdata++
	; Configure PWM5 MID value duty = adcdata
no0:	mov _PWMCFG,#0x05             ; Point to PWM5 MID LSB
	mov _PWMDATA,A                ; Set PWM MID LSB
        pop ACC
        pop dph
        pop dpl
        pop psw
  clr P2.0
        reti
_endasm;
}



